//global variables
var mtUrlnav = null;

/*********************************************************************************
 *  function:     onUrlnavLoad
 *	description:  window load function called from event listener
 ********************************************************************************/
function onUrlnavLoad() 
{
  //give the browser a chance to load completely
  setTimeout("delayedUrlnavLoad()",50);  
}

/*********************************************************************************
 *  function:     delayedUrlnavLoad
 *	description:  window load function called from timer set on windows load
 ********************************************************************************/
function delayedUrlnavLoad() {

  //create global urlnav object
  mtUrlnav = new urlnav();
  
  //override toolbox customize done function
  var toolbox = document.getElementById("navigator-toolbox");
  toolbox.customizeDone = onCustomizeDone;
}

/*********************************************************************************
 *  function:     onUrlnavUnload
 *	description:  window unload function called from event listener
 ********************************************************************************/
function onUrlnavUnload() 
{
  //destroy the urlnav object
  mtUrlnav.destroy();
  mtUrlnav = null;
  
  //return toolbox customize done function
  var toolbox = document.getElementById("navigator-toolbox");
  toolbox.customizeDone = BrowserToolboxCustomizeDone;  
}

/*********************************************************************************
 *  function:     onCustomizeDone
 *	description:  function called from customize toolbar
 ********************************************************************************/
function onCustomizeDone(aToolboxChanged)
{   
  //enable or disable based on if we are on the toolbar
  if (document.getElementById("urlnav-item"))
    mtUrlnav.enabled = true;
  else
    mtUrlnav.enabled = false;
    
  //reset buttons if on toolbar and was not before
  if (mtUrlnav.enabled)
  {  
    //set buttons
    if(mtUrlnav.prefs["urlnav.capture.remember"].value && mtUrlnav.prefs["urlnav.capture.url"].value != "")
	  mtUrlnav.capture(mtUrlnav.prefs["urlnav.capture.url"].value);
	else
	  mtUrlnav.release();
  }
  else
    mtUrlnav.release();
    
  //call browser customize done function
  BrowserToolboxCustomizeDone();
}

/*********************************************************************************
 *  function:     runSlideshow
 *	description:  called from timer to run slideshow.  Hack because I can not
 *                call object function
 ********************************************************************************/
function runSlideshow() 
{
  if (mtUrlnav.enabled && mtUrlnav.slideshow)
  {
    if (mtUrlnav.prefs["urlnav.slideshow.direction"].value == 0) 
      mtUrlnav.nav(document.getElementById("urlnav-up"));
    else
      mtUrlnav.nav(document.getElementById("urlnav-down"));   
  }
}

/*********************************************************************************
 *  function:     urlnavPref
 *	description:  function to create urlnav preference object
 ********************************************************************************/
function urlnavPref(aValue, aType) 
{
  this.value = aValue;
  this.type = aType;
}

/*********************************************************************************
 *  function:     urlnav
 *	description:  function to create urlnav object
 ********************************************************************************/
function urlnav() 
{
  this.init();
}

urlnav.prototype = 
{
  domain: "urlnav.",
  prefBranch: null,
  prefs: null,
  slideshow: false,
  enabled: false,
  
  init: function() 
  {
    // initialize default preferences
    this.prefs = new Object();
    this.prefs["urlnav.hide"] = new urlnavPref(true,"bool");  
    this.prefs["urlnav.history.updwn"] = new urlnavPref(false,"bool");   
    this.prefs["urlnav.history.return"] = new urlnavPref(false,"bool");
    this.prefs["urlnav.capture.remember"] = new urlnavPref(false,"bool");
    this.prefs["urlnav.capture.url"] = new urlnavPref("","char");
    this.prefs["urlnav.increment"] = new urlnavPref(1,"int");
    this.prefs["urlnav.slideshow.direction"] = new urlnavPref(0,"int");
    this.prefs["urlnav.slideshow.delay"] = new urlnavPref(3,"int");
            
    //setup preference service
    var prefService = "@mozilla.org/preferences-service;1";
    prefService = Components.classes[prefService].getService(Components.interfaces.nsIPrefService); 
    this.prefBranch = prefService.getBranch(null);
    
    //set preferences
    for (var i in this.prefs) 
    {
      this._setPref(i,true);
    }
    
    //add preference observer
    var pbi = this.prefBranch.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
    pbi.addObserver(this.domain, this, false);

    //set up load observer for slideshow
    var oObserver = "@mozilla.org/observer-service;1";
    oObserver = Components.classes[oObserver].getService();
    oObserver = oObserver.QueryInterface(Components.interfaces.nsIObserverService);
    oObserver.addObserver(this, "EndDocumentLoad", false);
    
    
    //set enabled if on toolbar
    if (document.getElementById("urlnav-item"))
      this.enabled = true;
    else
      this.enabled = false;
     
    if (this.enabled)
    { 
      //set buttons
      if(this.prefs["urlnav.capture.remember"].value && this.prefs["urlnav.capture.url"].value != "")
	    this.capture(this.prefs["urlnav.capture.url"].value);
	  else
	    this.release();
	}
  },
  
  destroy: function() 
  {
    //remove preference observer
    var pbi = this.prefBranch.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
    pbi.removeObserver(this.domain, this);
    
    //remove load observer for slideshow
    var oObserver = "@mozilla.org/observer-service;1";
    oObserver = Components.classes[oObserver].getService();
    oObserver = oObserver.QueryInterface(Components.interfaces.nsIObserverService);
    oObserver.removeObserver(this, "EndDocumentLoad");
   
    //destroy variables
    this.domain = null;
    this.prefBranch = null;
    this.prefs = null;
    this.slideshow = null;
    this.enabled = null;
  },

  release: function() 
  {
    if(this.prefs["urlnav.capture.remember"].value)
      this.prefBranch.setCharPref("urlnav.capture.url","");  
 
    this._setButtons("","release");
  },
   
  capture: function(aURL) 
  { 
    if(this.prefs["urlnav.capture.remember"].value)
      this.prefBranch.setCharPref("urlnav.capture.url",aURL);   
   
    this._setButtons(aURL,"capture");
  },
  
  nav: function(aEl) 
  {
    var url = aEl.getAttribute("url"); 
    
    this._setButtons(url,"nav");
    
    if (!this._inHistory(url))
      getBrowser().loadURI(url);
  },
  
  toggleSlideshow: function (aEl)
  {
    if (aEl.getAttribute("checked") != "true") 
    {
      this.slideshow = true;
      aEl.setAttribute("tooltiptext",aEl.getAttribute("enabledTooltip"));
      if (this.prefs["urlnav.slideshow.direction"].value == 0) 
        this.nav(document.getElementById("urlnav-up"));
      else
        this.nav(document.getElementById("urlnav-down"));   
    }
    else
    {
      this.slideshow = false;
      aEl.setAttribute("tooltiptext",aEl.getAttribute("disabledTooltip"));
    }
  },  
  
  observe: function(aSubject, aTopic, aPrefName) 
  {
    //if end document run slideshow
    if (aTopic == "EndDocumentLoad" && this.slideshow && aSubject == getBrowser().contentWindow)
      setTimeout("runSlideshow()",this.prefs["urlnav.slideshow.delay"].value*1000);    
    else 
    {
      this._setPref(aPrefName,false);
      switch (aPrefName) {
        case "urlnav.capture.url":
          if (this.enabled)
          {
            //set buttons
        	if(this.prefs["urlnav.capture.url"].value == "")
	      	  this.release();
	    	else
	      	  this.capture(this.prefs["urlnav.capture.url"].value); 
	      }
	      break;
	    case "urlnav.hide":
	      if (this.enabled)   
          {    
            var els = document.getElementById("urlnav-item").childNodes;
            for (var x = 0; x < els.length; ++x) 
            {
              if (els[x].hasAttribute("disabled")) 
              {
                if (this.prefs[aPrefName].value)
                  els[x].setAttribute("collapsed","true");
                else
                  els[x].removeAttribute("collapsed");
              }
            }
          }
          break;
      }
	} 
  },
      
  _inHistory: function(aURL) 
  {
    var hist = getBrowser().contentWindow.history;
    var url = document.getElementById("urlnav-return").getAttribute("url");
    var i, curPos, urlPos;
    
    //if rewind and use history true continue
    if (url == aURL && !this.prefs["urlnav.history.return"].value)
      return false;
    
    // if up-down and use history true continue     
    if (url != aURL && !this.prefs["urlnav.history.updwn"].value)
      return false;
      
    curPos = "";
    urlPos = "";
    
    // get position of current in history
    for (i = 0; i < hist.length; ++i) 
    {
      if (hist[i] == hist.current) 
      {
        curPos = i;
        break;
      }
    }  
    
    // get position of url in history
    for (i = 0; i < hist.length; ++i) 
    {
      if (hist[i] == aURL) 
      {
        urlPos = i;
        break;
      }
    } 
    
    if(curPos == "" || urlPos == "")
      return false;
    
    hist.go(urlPos-curPos);
    return true;        
  },
  
  _increment: function (aString, amount) 
  {
    var endIndex, startIndex, len, oldNum, newNum;
    len = aString.length;
    
    for (endIndex = len-1; endIndex >=0; --endIndex)
      if (this._isDigit(aString.charAt(endIndex))) 
      {
        for(startIndex = endIndex-1; startIndex >=0; --startIndex)
          if (!this._isDigit(aString.charAt(startIndex)))
            break;
        break;
      }
    
    ++startIndex;
    
    if (endIndex < 0)
      return aString;
    
    oldNum = aString.substring(startIndex, endIndex+1);
    newNum = '' + (parseInt(oldNum,10) + amount);
    while (newNum.length < oldNum.length)
      newNum = '0' + newNum;
    
    return aString.substring(0,startIndex) + newNum + aString.slice(endIndex+1);
  },
  
  _setButtons: function (aURL, aType)
  {
    var el, down, up;
    if (this.enabled)
    {
      switch (aType)
      {
        case "release":
          //capture button
          el = document.getElementById("urlnav-capture");
          el.removeAttribute("captured");
            
          //release button
          el = document.getElementById("urlnav-release");
          el.setAttribute("disabled","true");
          if (this.prefs["urlnav.hide"].value)
            el.setAttribute("collapsed","true");
             
          //down button
    	  el = document.getElementById("urlnav-down");
    	  el.setAttribute("tooltiptext",el.getAttribute("disabledTooltip")); 
		  el.removeAttribute("url");
    	  el.setAttribute("disabled","true");
          if (this.prefs["urlnav.hide"].value)
            el.setAttribute("collapsed","true");
    
    	  //up button
    	  el = document.getElementById("urlnav-up");
    	  el.setAttribute("tooltiptext",el.getAttribute("disabledTooltip")); 
		  el.removeAttribute("url");
    	  el.setAttribute("disabled","true");
          if (this.prefs["urlnav.hide"].value)
            el.setAttribute("collapsed","true");  
     
    	  //return button
    	  el = document.getElementById("urlnav-return");
    	  el.setAttribute("tooltiptext",el.getAttribute("disabledTooltip")); 
		  el.removeAttribute("url");
    	  el.setAttribute("disabled","true");
          if (this.prefs["urlnav.hide"].value)
            el.setAttribute("collapsed","true");
        
    	  //slideshow button
    	  this.slideshow = false;
    	  el = document.getElementById("urlnav-slideshow");
    	  el.setAttribute("tooltiptext",el.getAttribute("disabledTooltip")); 
          el.removeAttribute("checked");
    	  el.setAttribute("disabled","true");
          if (this.prefs["urlnav.hide"].value)
            el.setAttribute("collapsed","true");
          break;
          
        case "capture":
          down = this._increment(aURL,-1*this.prefs["urlnav.increment"].value);
          up = this._increment(aURL,this.prefs["urlnav.increment"].value);
          
          //capture button
          el = document.getElementById("urlnav-capture");
          el.setAttribute("captured","true");
                    
          //release button
          el = document.getElementById("urlnav-release");
          el.removeAttribute("disabled");
          el.removeAttribute("collapsed");
             
          //down button
    	  el = document.getElementById("urlnav-down");
          el.removeAttribute("disabled");
          el.removeAttribute("collapsed");
		  el.setAttribute("url",down);
          el.setAttribute("tooltiptext",el.getAttribute("enabledTooltip") + down);
          
    	  //up button
    	  el = document.getElementById("urlnav-up");
          el.removeAttribute("disabled");
          el.removeAttribute("collapsed");
		  el.setAttribute("url",up);
          el.setAttribute("tooltiptext",el.getAttribute("enabledTooltip") + up);
     
    	  //return button
    	  el = document.getElementById("urlnav-return");
          el.removeAttribute("disabled");
          el.removeAttribute("collapsed");
		  el.setAttribute("url",aURL);
          el.setAttribute("tooltiptext",el.getAttribute("enabledTooltip") + aURL);
        
    	  //slideshow button
    	  this.slideshow = false;
    	  el = document.getElementById("urlnav-slideshow");
          el.removeAttribute("disabled");
          el.removeAttribute("collapsed");
          el.removeAttribute("checked");          
    	  el.setAttribute("tooltiptext",el.getAttribute("disabledTooltip")); 
          break;
          
        case "nav":
          down = this._increment(aURL,-1*this.prefs["urlnav.increment"].value);
          up = this._increment(aURL,this.prefs["urlnav.increment"].value);
          //down button
    	  el = document.getElementById("urlnav-down");
          el.removeAttribute("disabled");
          el.removeAttribute("collapsed");
		  el.setAttribute("url",down);
          el.setAttribute("tooltiptext",el.getAttribute("enabledTooltip") + down);
          
    	  //up button
    	  el = document.getElementById("urlnav-up");
          el.removeAttribute("disabled");
          el.removeAttribute("collapsed");
		  el.setAttribute("url",up);
          el.setAttribute("tooltiptext",el.getAttribute("enabledTooltip") + up);
          break;
      }
    }                
  },
  
  _setPref: function(aPrefName, aForce)
  {
    try 
    {
      switch(this.prefs[aPrefName].type) 
      {
        case "bool":
          this.prefs[aPrefName].value = this.prefBranch.getBoolPref(aPrefName);
          break;
        case "int":
          this.prefs[aPrefName].value = this.prefBranch.getIntPref(aPrefName);
          break;
        default:
          this.prefs[aPrefName].value = this.prefBranch.getCharPref(aPrefName);
          break;
      }
    }
    catch(e) 
    {
      if (aForce) 
      {  
        switch(this.prefs[aPrefName].type) {
          case "bool":
            this.prefBranch.setBoolPref(aPrefName,this.prefs[aPrefName].value);
            break;
          case "int":
            this.prefBranch.setIntPref(aPrefName,this.prefs[aPrefName].value);
            break;
          default:
            this.prefBranch.setCharPref(aPrefName,this.prefs[aPrefName].value);
            break;
        }
      }
    }
  },

  _isDigit: function(c) 
  {
    var c_num = parseInt(c);
    return (0 <= c_num && c_num <= 9);
  }  
}